module AWS
  module EC2
    class Base < AWS::Base


      # Not yet implemented
      #
      # @todo Implement this method
      #
      def confirm_product_instance( options = {} )
        raise "Not yet implemented"
      end


    end
  end
end

